<?php

require_once('CssMin.php');
require_once('JavaScriptPacker.php');

class DynamicRes extends CWidget {
    public $urlConfig = array( // Configurar conversão Url To Path
        'baseUrl'  => 'http://meusite.com/',
        'basePath' => '/home/meusite/',
    );
    
    public $saved = false;
    
	private $jsList = array();
	private $cssList = array();
    
    public function getCachePath() {
        return Yii::app()->assetManager->basePath;
    }
    
    public function getUrlPath() {
        return Yii::app()->assetManager->baseUrl;
    }
    
    function urlToPath($url) { 
        $dS = DIRECTORY_SEPARATOR;
        $from = preg_quote($this->urlConfig['baseUrl'], '/');
        $to  = preg_quote(str_replace('\\' , '/' , $this->urlConfig['basePath']), '/');
        $result = preg_replace("/$from/", "/$to/", $url, 1);
        $result = str_replace('/' , $dS , str_replace('//','/',stripcslashes($result))); // fix windows
        if($result[0]=='\\') $result = substr($result, 1); // fix windows
        return $result;
    }
    
    function parserCss($url, $content) {
        $newContent = $content;
        $aux = explode('/',$url);
        $dir = preg_quote(implode('/', array_slice($aux,0,count($aux)-1)));
        $newContent = preg_replace('@url\(([^/])@',"url($dir/$1", $newContent);
        return $newContent;
    }
	
	private function registerFile(&$array, $name) {
		if(file_exists($this->urlToPath($name))) {
			$array[] = $name;
			return true;
		} else return false;
	}
	
	public function registerCssFile($urlName) {
		return $this->registerFile($this->cssList, $urlName);
	}
	
	public function registerScriptFile($urlName) {
		return $this->registerFile($this->jsList, $urlName);
	}
	
	private function saveFileScheme(&$array, $extension) {
		$virtualFiles = array();
		foreach($array as $name) {
            $realName = $this->urlToPath($name);
			$content = file_get_contents($realName);
			$hash = md5($content);
			$nomeAux = explode(DIRECTORY_SEPARATOR, $name);
			$virtualFiles[] = array(
                'urlName' => $name,
                'pathName' => $realName,
				'content' => $content,
				'hash' => $hash,
				'nome' => end($nomeAux),
                'extension' => strtolower($extension),
			);
		}
		$generatedName = $this->generateName($virtualFiles, $extension);
		$name = $this->getCachePath() . DIRECTORY_SEPARATOR . $generatedName;
		$urlName = $this->getUrlPath() . '/' . $generatedName;
		if(!file_exists($name)) {
			$this->generateFile($virtualFiles, $name);
			return $urlName;
		} else return $urlName; 
	}
	
	private function generateFile($virtualFiles, $name) {
		$finalContent = '';
		foreach($virtualFiles as $virtualFile) {
            $hoje = utf8_decode(date('d/m/y \á\s H:i:s'));
			$finalContent .= "/* START OF FILE {$virtualFile['nome']} (Comprimido $hoje)------------ */ \n";
            $thisContent = $this->compressContent($virtualFile['content'], $virtualFile['extension']);
            if($virtualFile['extension']=='css') $thisContent = $this->parserCss($virtualFile['urlName'], $thisContent);
			$finalContent .= $thisContent;
			$finalContent .= "\n/* END OF FILE {$virtualFile['nome']} ------------------------ */\n";
		}
        
		file_put_contents($name, $finalContent);
		return $name;
	}
	
	private function generateName($virtualFiles, $extension) {
		$total_hash = '';
		foreach($virtualFiles as $array) $total_hash .= $array['hash'];
		return md5($total_hash) . ".$extension";
	}
	
	private function compressContent($content, $type) {
        if($type=='css') {
            return CssMin::minify($content, array(), array('CompressUnitValues' => true));
        } else if($type=='js') { // is js
            $packer = new JavaScriptPacker($content, 'Normal', true, false);
            return $packer->pack();
        } else {
            //dont Compress
            return $content;
        }
		
	}
	
	public function saveScheme() {
        if(count($this->jsList)>0) {
            $fileJs = $this->saveFileScheme($this->jsList, 'js');
            Yii::app()->clientScript->registerScriptFile($fileJs);
        }
        if(count($this->cssList)>0) {
            $fileCss = $this->saveFileScheme($this->cssList, 'css');
            Yii::app()->clientScript->registerCssFile($fileCss);
        }
        $this->jsList = array();
        $this->cssList = array();
	}
}
